require 'composable_fixtures/fixture_set_definition'
require 'composable_fixtures/fixture_set_repository'
require 'composable_fixtures/fixture_directory'
require 'composable_fixtures/fixture_composer'

module ComposableFixtures
  module ExampleGroupHelpers
    def use_composed_fixture_set(*fixtures)
      unless dir = FixtureDirectory.get(fixtures)
        dir = FixtureDirectory.create(fixtures)
        dir.write(
          FixtureComposer.new(fixtures).as_filename_to_yaml
        )
      end
      
      self.fixture_path = dir.path
    end

    def self.extended(base)
      RSpec.configure do |config|
        config.after(:suite) do
          FixtureDirectory.destroy_all
        end
      end
    end
  end

  def self.define(&block)
    module_eval(&block)
  end

  def self.set(name, *fixture_set_paths)
    FixtureSetDefinition.create(name, fixture_set_paths)
  end

  def self.compose(name, *set_names)
    fixture_set_paths =
      set_names.reduce([]) do |fixture_set_paths, set_name|
        fixture_set_paths += FixtureSetRepository.instance.fetch(set_name).paths
      end
    set(name, *fixture_set_paths)
  end
end
