require 'tmpdir'
require 'singleton'

module ComposableFixtures
  class FixtureDirectory
    class DirectoryAlreadyExistsError < StandardError; end
    class DirectoryDoesntExistError   < StandardError; end

    def initialize(name)
      @dir = Dir.mktmpdir(name)
    end

    def destroy
      raise DirectoryDoesntExistError unless @dir
      FileUtils.remove_entry @dir
    end

    def write(name_to_fixtures)
      name_to_fixtures.each do |(name, yaml)|
        File.open(File.join(path, name), "w") do |f|
          f.write yaml
        end
      end      
    end

    def path
      @dir.to_s
    end

    def self.get(fixtures)
      key = repository_key(fixtures)
      repository.fetch(key, nil)
    end

    def self.create(fixtures)
      key = repository_key(fixtures)
      raise DirectoryAlreadyExistsError if @@repository.has_key?(key)

      repository[key] = new(key)
    end

    def self.destroy_all
      repository.each do |key, dir|
        dir.destroy
      end
    end

    def self.repository_key(fixtures)
      fixtures.join('-')
    end

    def self.repository
      @@repository ||= {}
    end
  end
end
